################################################################################
#-------------------------------------------------------------------------------
# Name:      BaggingGLM for R
# Purpose:   Analysis of zero-inflated data (presence/absence records)
# Reference: Ecological Informatics 6:270-275.
#
# Author:      Takeshi OSAWA <arosawa@gmail.com>
#
# Created:     19/08/2011
# Lat modified:    19/08/2011
#
# Copyright:   (c) Takeshi OSAWA 2011
# Licence:     Creative Commons
#-------------------------------------------------------------------------------
################################################################################

BaggingGLM <- function(object,num,criteria){

 ## preparations ##
 # extract model variables
	terms <- object$terms
	label <- attr(terms, "dataClasses")
	label <- attr(label, "names")

 # extract present/absent records ݕs݃f[^𔲂o
	all_data <- object$data
	eval(parse(text= paste("present <- all_data[all_data$",label[1]," ==1, ]",sep="")))
	eval(parse(text= paste("absent <- all_data[all_data$",label[1]," ==0, ]",sep="")))

 # set object
	COEF <- NULL

 ## start bootstrapping ##
	for(i in 1:num){
	# extract absent records
		zeronum <- NROW(present) * 1  # extract absent records (same number of present) ݃f[^Ɠ ̒lςΉ{łΉł
		analysis <- NULL
		for(j in 1:zeronum){
			rand <- sample(NROW(absent)) # generate random number
			tmp <- absent[rand[1],]  # pick up one record randomly
 			analysis <- rbind(analysis,tmp)
		}

 # set one data set for learning
		analysis1 <- rbind(analysis, present)

 # fitting GLM
	# combine explanation variables
		exp_val <- label[2]
		for(k in 3:NROW(label)){
			exp_val <- paste(exp_val,"+", label[k])
		}
		obj <- eval(parse(text=paste("glm(", label[1], "~", exp_val,", data = analysis1, family = binomial(logit) )",sep="")))

 # store coefficients
		COEF[i] <- list(obj$coefficients)
		obj <- NULL
	}

 ## model ensemble ##

#------- sub_func --------#
cutting <- function(x,y){  # 12ʉʂ؂֐
	tmp <- x[-(1:y)]  # ォJbg
	tmp <- tmp[-((NROW(tmp) - y):NROW(tmp))]  # Jbg
	return(tmp)
}
#-------------------------#

 # averaging GLM models
	param <- NULL
	for(l in 1:NROW(COEF)){
		param <- rbind(param,COEF[[l]])
		params <- as.data.frame(param)
	}

	result_all <- NULL
	se_all <- NULL
	if(criteria == 1){ # 90% confidence interval
		for(m in 1:NCOL(params)){
			tmp <- params[,m]
			tmp1 <- cutting(tmp,0.5)
			result_all[m] <- mean(tmp1)
			se_all[m] <- sd(tmp1)/sqrt(length(tmp1))
		}
	}
	else if(criteria == 2){ # 95% confidence interval
		for(m in 1:NCOL(params)){
			tmp <- params[,m]
			tmp1 <- cutting(tmp,0.25)
			result_all[m] <- mean(tmp1)
			se_all[m] <- sd(tmp1)/sqrt(length(tmp1))
		}
	}
	else if(criteria == 3){ # average of all
		for(m in 1:NCOL(params)){
			tmp <- params[,m]
			result_all[m] <- mean(tmp)
			se_all[m] <- sd(tmp)/sqrt(length(tmp))
		}
	}

	result_fin <- data.frame(names(params),result_all,se_all)
	names(result_fin) <- c("Coefficients:","Estimated values","S.E")
	return(result_fin)
}

